@@ -8,9 +8,8 @@ from functools import reduce
 from django.core.exceptions import ImproperlyConfigured
 from django.db import models
 from django.db.models.constants import LOOKUP_SEP
-from django.db.models.sql.constants import ORDER_PATTERN
 from django.template import loader
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 from django.utils.translation import gettext_lazy as _
 
 from rest_framework.compat import coreapi, coreschema, distinct
@@ -86,7 +85,7 @@ class SearchFilter(BaseFilterBackend):
                 search_field = search_field[1:]
             # Annotated fields do not need to be distinct
             if isinstance(queryset, models.QuerySet) and search_field in queryset.query.annotations:
-                return False
+                continue
             parts = search_field.split(LOOKUP_SEP)
             for part in parts:
                 field = opts.get_field(part)
@@ -97,6 +96,9 @@ class SearchFilter(BaseFilterBackend):
                     if any(path.m2m for path in path_info):
                         # This field is a m2m relation so we know we need to call distinct
                         return True
+                else:
+                    # This field has a custom __ query transform but is not a relational field.
+                    break
         return False
 
     def filter_queryset(self, request, queryset, view):
@@ -151,8 +153,8 @@ class SearchFilter(BaseFilterBackend):
                 required=False,
                 location='query',
                 schema=coreschema.String(
-                    title=force_text(self.search_title),
-                    description=force_text(self.search_description)
+                    title=force_str(self.search_title),
+                    description=force_str(self.search_description)
                 )
             )
         ]
@@ -163,7 +165,7 @@ class SearchFilter(BaseFilterBackend):
                 'name': self.search_param,
                 'required': False,
                 'in': 'query',
-                'description': force_text(self.search_description),
+                'description': force_str(self.search_description),
                 'schema': {
                     'type': 'string',
                 },
@@ -256,7 +258,13 @@ class OrderingFilter(BaseFilterBackend):
 
     def remove_invalid_fields(self, queryset, fields, view, request):
         valid_fields = [item[0] for item in self.get_valid_fields(queryset, view, {'request': request})]
-        return [term for term in fields if term.lstrip('-') in valid_fields and ORDER_PATTERN.match(term)]
+
+        def term_valid(term):
+            if term.startswith("-"):
+                term = term[1:]
+            return term in valid_fields
+
+        return [term for term in fields if term_valid(term)]
 
     def filter_queryset(self, request, queryset, view):
         ordering = self.get_ordering(request, queryset, view)
@@ -295,8 +303,8 @@ class OrderingFilter(BaseFilterBackend):
                 required=False,
                 location='query',
                 schema=coreschema.String(
-                    title=force_text(self.ordering_title),
-                    description=force_text(self.ordering_description)
+                    title=force_str(self.ordering_title),
+                    description=force_str(self.ordering_description)
                 )
             )
         ]
@@ -307,7 +315,7 @@ class OrderingFilter(BaseFilterBackend):
                 'name': self.ordering_param,
                 'required': False,
                 'in': 'query',
-                'description': force_text(self.ordering_description),
+                'description': force_str(self.ordering_description),
                 'schema': {
                     'type': 'string',
                 },
