@@ -9,7 +9,7 @@ from urllib import parse
 from django.core.paginator import InvalidPage
 from django.core.paginator import Paginator as DjangoPaginator
 from django.template import loader
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 from django.utils.translation import gettext_lazy as _
 
 from rest_framework.compat import coreapi, coreschema
@@ -80,7 +80,7 @@ def _get_displayed_page_numbers(current, final):
 
     # Now sort the page numbers and drop anything outside the limits.
     included = [
-        idx for idx in sorted(list(included))
+        idx for idx in sorted(included)
         if 0 < idx <= final
     ]
 
@@ -138,6 +138,9 @@ class BasePagination:
     def get_paginated_response(self, data):  # pragma: no cover
         raise NotImplementedError('get_paginated_response() must be implemented.')
 
+    def get_paginated_response_schema(self, schema):
+        return schema
+
     def to_html(self):  # pragma: no cover
         raise NotImplementedError('to_html() must be implemented to display page controls.')
 
@@ -222,6 +225,32 @@ class PageNumberPagination(BasePagination):
             ('results', data)
         ]))
 
+    def get_paginated_response_schema(self, schema):
+        return {
+            'type': 'object',
+            'properties': {
+                'count': {
+                    'type': 'integer',
+                    'example': 123,
+                },
+                'next': {
+                    'type': 'string',
+                    'nullable': True,
+                    'format': 'uri',
+                    'example': 'http://api.example.org/accounts/?{page_query_param}=4'.format(
+                        page_query_param=self.page_query_param)
+                },
+                'previous': {
+                    'type': 'string',
+                    'nullable': True,
+                    'format': 'uri',
+                    'example': 'http://api.example.org/accounts/?{page_query_param}=2'.format(
+                        page_query_param=self.page_query_param)
+                },
+                'results': schema,
+            },
+        }
+
     def get_page_size(self, request):
         if self.page_size_query_param:
             try:
@@ -286,7 +315,7 @@ class PageNumberPagination(BasePagination):
                 location='query',
                 schema=coreschema.Integer(
                     title='Page',
-                    description=force_text(self.page_query_description)
+                    description=force_str(self.page_query_description)
                 )
             )
         ]
@@ -298,7 +327,7 @@ class PageNumberPagination(BasePagination):
                     location='query',
                     schema=coreschema.Integer(
                         title='Page size',
-                        description=force_text(self.page_size_query_description)
+                        description=force_str(self.page_size_query_description)
                     )
                 )
             )
@@ -310,7 +339,7 @@ class PageNumberPagination(BasePagination):
                 'name': self.page_query_param,
                 'required': False,
                 'in': 'query',
-                'description': force_text(self.page_query_description),
+                'description': force_str(self.page_query_description),
                 'schema': {
                     'type': 'integer',
                 },
@@ -322,7 +351,7 @@ class PageNumberPagination(BasePagination):
                     'name': self.page_size_query_param,
                     'required': False,
                     'in': 'query',
-                    'description': force_text(self.page_size_query_description),
+                    'description': force_str(self.page_size_query_description),
                     'schema': {
                         'type': 'integer',
                     },
@@ -369,6 +398,32 @@ class LimitOffsetPagination(BasePagination):
             ('results', data)
         ]))
 
+    def get_paginated_response_schema(self, schema):
+        return {
+            'type': 'object',
+            'properties': {
+                'count': {
+                    'type': 'integer',
+                    'example': 123,
+                },
+                'next': {
+                    'type': 'string',
+                    'nullable': True,
+                    'format': 'uri',
+                    'example': 'http://api.example.org/accounts/?{offset_param}=400&{limit_param}=100'.format(
+                        offset_param=self.offset_query_param, limit_param=self.limit_query_param),
+                },
+                'previous': {
+                    'type': 'string',
+                    'nullable': True,
+                    'format': 'uri',
+                    'example': 'http://api.example.org/accounts/?{offset_param}=200&{limit_param}=100'.format(
+                        offset_param=self.offset_query_param, limit_param=self.limit_query_param),
+                },
+                'results': schema,
+            },
+        }
+
     def get_limit(self, request):
         if self.limit_query_param:
             try:
@@ -478,7 +533,7 @@ class LimitOffsetPagination(BasePagination):
                 location='query',
                 schema=coreschema.Integer(
                     title='Limit',
-                    description=force_text(self.limit_query_description)
+                    description=force_str(self.limit_query_description)
                 )
             ),
             coreapi.Field(
@@ -487,7 +542,7 @@ class LimitOffsetPagination(BasePagination):
                 location='query',
                 schema=coreschema.Integer(
                     title='Offset',
-                    description=force_text(self.offset_query_description)
+                    description=force_str(self.offset_query_description)
                 )
             )
         ]
@@ -498,7 +553,7 @@ class LimitOffsetPagination(BasePagination):
                 'name': self.limit_query_param,
                 'required': False,
                 'in': 'query',
-                'description': force_text(self.limit_query_description),
+                'description': force_str(self.limit_query_description),
                 'schema': {
                     'type': 'integer',
                 },
@@ -507,7 +562,7 @@ class LimitOffsetPagination(BasePagination):
                 'name': self.offset_query_param,
                 'required': False,
                 'in': 'query',
-                'description': force_text(self.offset_query_description),
+                'description': force_str(self.offset_query_description),
                 'schema': {
                     'type': 'integer',
                 },
@@ -840,6 +895,22 @@ class CursorPagination(BasePagination):
             ('results', data)
         ]))
 
+    def get_paginated_response_schema(self, schema):
+        return {
+            'type': 'object',
+            'properties': {
+                'next': {
+                    'type': 'string',
+                    'nullable': True,
+                },
+                'previous': {
+                    'type': 'string',
+                    'nullable': True,
+                },
+                'results': schema,
+            },
+        }
+
     def get_html_context(self):
         return {
             'previous_url': self.get_previous_link(),
@@ -861,7 +932,7 @@ class CursorPagination(BasePagination):
                 location='query',
                 schema=coreschema.String(
                     title='Cursor',
-                    description=force_text(self.cursor_query_description)
+                    description=force_str(self.cursor_query_description)
                 )
             )
         ]
@@ -873,7 +944,7 @@ class CursorPagination(BasePagination):
                     location='query',
                     schema=coreschema.Integer(
                         title='Page size',
-                        description=force_text(self.page_size_query_description)
+                        description=force_str(self.page_size_query_description)
                     )
                 )
             )
@@ -885,7 +956,7 @@ class CursorPagination(BasePagination):
                 'name': self.cursor_query_param,
                 'required': False,
                 'in': 'query',
-                'description': force_text(self.cursor_query_description),
+                'description': force_str(self.cursor_query_description),
                 'schema': {
                     'type': 'integer',
                 },
@@ -897,7 +968,7 @@ class CursorPagination(BasePagination):
                     'name': self.page_size_query_param,
                     'required': False,
                     'in': 'query',
-                    'description': force_text(self.page_size_query_description),
+                    'description': force_str(self.page_size_query_description),
                     'schema': {
                         'type': 'integer',
                     },
