@@ -4,6 +4,7 @@ from importlib import reload as reload_module
 import pytest
 from django.core.exceptions import ImproperlyConfigured
 from django.db import models
+from django.db.models import CharField, Transform
 from django.db.models.functions import Concat, Upper
 from django.test import TestCase
 from django.test.utils import override_settings
@@ -189,6 +190,41 @@ class SearchFilterTests(TestCase):
 
         assert terms == ['asdf']
 
+    def test_search_field_with_additional_transforms(self):
+        from django.test.utils import register_lookup
+
+        class SearchListView(generics.ListAPIView):
+            queryset = SearchFilterModel.objects.all()
+            serializer_class = SearchFilterSerializer
+            filter_backends = (filters.SearchFilter,)
+            search_fields = ('text__trim', )
+
+        view = SearchListView.as_view()
+
+        # an example custom transform, that trims `a` from the string.
+        class TrimA(Transform):
+            function = 'TRIM'
+            lookup_name = 'trim'
+
+            def as_sql(self, compiler, connection):
+                sql, params = compiler.compile(self.lhs)
+                return "trim(%s, 'a')" % sql, params
+
+        with register_lookup(CharField, TrimA):
+            # Search including `a`
+            request = factory.get('/', {'search': 'abc'})
+
+            response = view(request)
+            assert response.data == []
+
+            # Search excluding `a`
+            request = factory.get('/', {'search': 'bc'})
+            response = view(request)
+            assert response.data == [
+                {'id': 1, 'title': 'z', 'text': 'abc'},
+                {'id': 2, 'title': 'zz', 'text': 'bcd'},
+            ]
+
 
 class AttributeModel(models.Model):
     label = models.CharField(max_length=32)
@@ -368,6 +404,21 @@ class SearchFilterAnnotatedFieldTests(TestCase):
         assert len(response.data) == 1
         assert response.data[0]['title_text'] == 'ABCDEF'
 
+    def test_must_call_distinct_subsequent_m2m_fields(self):
+        f = filters.SearchFilter()
+
+        queryset = SearchFilterModelM2M.objects.annotate(
+            title_text=Upper(
+                Concat(models.F('title'), models.F('text'))
+            )
+        ).all()
+
+        # Sanity check that m2m must call distinct
+        assert f.must_call_distinct(queryset, ['attributes'])
+
+        # Annotated field should not prevent m2m must call distinct
+        assert f.must_call_distinct(queryset, ['title_text', 'attributes'])
+
 
 class OrderingFilterModel(models.Model):
     title = models.CharField(max_length=20, verbose_name='verbose title')
