@@ -311,7 +311,8 @@ class TestMetadata:
 class TestSimpleMetadataFieldInfo(TestCase):
     def test_null_boolean_field_info_type(self):
         options = metadata.SimpleMetadata()
-        field_info = options.get_field_info(serializers.NullBooleanField())
+        field_info = options.get_field_info(serializers.BooleanField(
+            allow_null=True))
         assert field_info['type'] == 'boolean'
 
     def test_related_field_choices(self):
