@@ -2,13 +2,14 @@ import unittest
 
 from django.template import Context, Template
 from django.test import TestCase
+from django.utils.html import urlize
 
 from rest_framework.compat import coreapi, coreschema
 from rest_framework.relations import Hyperlink
 from rest_framework.templatetags import rest_framework
 from rest_framework.templatetags.rest_framework import (
     add_nested_class, add_query_param, as_string, break_long_headers,
-    format_value, get_pagination_html, schema_links, urlize_quoted_links
+    format_value, get_pagination_html, schema_links
 )
 from rest_framework.test import APIRequestFactory
 
@@ -246,7 +247,7 @@ class Issue1386Tests(TestCase):
 
     def test_issue_1386(self):
         """
-        Test function urlize_quoted_links with different args
+        Test function urlize with different args
         """
         correct_urls = [
             "asdf.com",
@@ -255,7 +256,7 @@ class Issue1386Tests(TestCase):
             "as.d8f.ghj8.gov",
         ]
         for i in correct_urls:
-            res = urlize_quoted_links(i)
+            res = urlize(i)
             self.assertNotEqual(res, i)
             self.assertIn(i, res)
 
@@ -264,11 +265,11 @@ class Issue1386Tests(TestCase):
             "asdf.netnet",
         ]
         for i in incorrect_urls:
-            res = urlize_quoted_links(i)
+            res = urlize(i)
             self.assertEqual(i, res)
 
         # example from issue #1386, this shouldn't raise an exception
-        urlize_quoted_links("asdf:[/p]zxcv.com")
+        urlize("asdf:[/p]zxcv.com")
 
     def test_smart_urlquote_wrapper_handles_value_error(self):
         def mock_smart_urlquote(url):
@@ -289,7 +290,10 @@ class URLizerTests(TestCase):
         For all items in dict test assert that the value is urlized key
         """
         for original, urlized in data.items():
-            assert urlize_quoted_links(original, nofollow=False) == urlized
+            print('====')
+            print(repr(urlize(original, nofollow=False)))
+            print(repr(urlized))
+            assert urlize(original, nofollow=False) == urlized
 
     def test_json_with_url(self):
         """
@@ -297,26 +301,26 @@ class URLizerTests(TestCase):
         """
         data = {}
         data['"url": "http://api/users/1/", '] = \
-            '&quot;url&quot;: &quot;<a href="http://api/users/1/">http://api/users/1/</a>&quot;, '
+            '"url": "<a href="http://api/users/1/">http://api/users/1/</a>", '
         data['"foo_set": [\n    "http://api/foos/1/"\n], '] = \
-            '&quot;foo_set&quot;: [\n    &quot;<a href="http://api/foos/1/">http://api/foos/1/</a>&quot;\n], '
+            '"foo_set": [\n    "<a href="http://api/foos/1/">http://api/foos/1/</a>"\n], '
         self._urlize_dict_check(data)
 
     def test_template_render_with_autoescape(self):
         """
         Test that HTML is correctly escaped in Browsable API views.
         """
-        template = Template("{% load rest_framework %}{{ content|urlize_quoted_links }}")
+        template = Template("{% load rest_framework %}{{ content|urlize }}")
         rendered = template.render(Context({'content': '<script>alert()</script> http://example.com'}))
         assert rendered == '&lt;script&gt;alert()&lt;/script&gt;' \
                            ' <a href="http://example.com" rel="nofollow">http://example.com</a>'
 
     def test_template_render_with_noautoescape(self):
         """
-        Test if the autoescape value is getting passed to urlize_quoted_links filter.
+        Test if the autoescape value is getting passed to urlize filter.
         """
         template = Template("{% load rest_framework %}"
-                            "{% autoescape off %}{{ content|urlize_quoted_links }}"
+                            "{% autoescape off %}{{ content|urlize }}"
                             "{% endautoescape %}")
         rendered = template.render(Context({'content': '<b> "http://example.com" </b>'}))
         assert rendered == '<b> "<a href="http://example.com" rel="nofollow">http://example.com</a>" </b>'
